/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim.simulation;

import java.util.Hashtable;
import java.util.Iterator;

/**
 * Zustand eines endlichen Automaten. Diese Implementierung speichert in einer 
 * Zustands-Instanz eine Hash-Tabelle mit Zustands-bergngen und einen Namen.
 * 
 * @version $Revision: 1.3 $ / $Date: 2008/10/31 13:30:31 $ / ($Author: Sans $)
 *
 * @author Dambach-Werke GmbH
 * @author Stefan Sans
 */    
public class Zustand 
{
  /**
   * Name des Zustands
   */        
  private String _name;
  
  /**
   * Hash-Tabelle, die die Zustands-bergnge speichert
   */        
  protected Hashtable<Integer, Zustand> _uebergangHash = new Hashtable<Integer, Zustand>();
  
  /**
   * Erzeuge neuen Zustand, mit Zustandsnamen
   * @param name Name des Zustands
   */        
  public Zustand(String name) 
  {
      _name = name;
  }
      
  /**
   * Setze den Namen des Zustands
   * @param name neuer Name des Zustands
   */        
  public void setName(String name) 
  {
    _name = name;
  }
  
  /**
   * Bestimmt den Namen des Zustands
   * @return Name des Zustands
   */
  public String getName() 
  {
    return _name;
  }
  
  /**
   * Fge neuen Zustands-bergang zu diesem Zustand hinzu  
   * @param uebergang Id fr den neuen bergang
   * @param endZustand End-Zustand des neuen bergangs
   */        
  public void hinzufuegenUebergang(int uebergang, Zustand endZustand) 
  {
    _uebergangHash.put(new Integer(uebergang), endZustand);            
  }

  /**
   * Realisiere einen bergang. Diese Methode ndert nichts an dieser 
   * Instanz, sondern gibt den Endzustand des zu relisierenden bergangs 
   * zurck.
   * @param uebergang die Id des zu relisierenden bergangs
   * @return den Endzustand des zu relisierenden bergangs odet null, 
   * falls der bergang nich definiert ist
   */
  public Zustand realisiereUebergang(int uebergang) 
  {
    return (Zustand)_uebergangHash.get(new Integer(uebergang));
  }

  /**
   * Berechnet fr einen gegebenen End-Zustand die Id des bergangs.
   *  
   * @param endZustand der End-Zustand des zu berechnenden bergangs
   * @return die Id des bergangs Id des bergangs, zu dem diese Instanz 
   * den Anfangszustand und der bergebene Zustand den Endzustand bildet
   * @throws UnsupportedOperationException falls ein bergang zum Endzustand 
   * fr diese Instanz nicht definiert ist
   */
  public int berechneUebergang(Zustand endZustand) 
    throws UnsupportedOperationException 
  {
    Iterator it = _uebergangHash.keySet().iterator();
    
    while (it.hasNext())
    {
      Integer key = (Integer)it.next();
      Zustand zustand = (Zustand)_uebergangHash.get(key);
      if (zustand == endZustand)
      {
        return key.intValue();
      }
    }
    
    throw new UnsupportedOperationException("Dieser bergang ist nicht realisierbar");
  }
  
  @Override        
  public String toString() 
  {
    return getName();
  }

}