/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 
package de.bsvrz.vew.sim.sim.steuerung;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.Data.TimeValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.sim.sim.datengenerator.DatenGeneratorOfflineSimulation;
import de.bsvrz.vew.sim.sim.simulation.OfflineSimulation;
import de.bsvrz.vew.sim.sim.simulation.Simulation;
import de.bsvrz.vew.sim.sim.simulation.Zustand;
import de.bsvrz.vew.sim.sim.simulation.Simulation.ZustandsBezeichner;

/**
 * Steuerung zu einer Offline-Simulation.
 *  
 * @version  $Revision: 1.1 $ / $Date: 2008/10/31 13:30:30 $ / ($Author: Sans $)
 * @author  Dambach-Werke GmbH
 * @author  Stefan Sans
 */
public class SteuerungOfflineSimulation extends SteuerungOnlineSimulation
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Geschwindigkeitsfaktor fr diese Simulation
   */
  private float _faktor;
  
  /**
   * Beginn dieser Simulation
   */
  private long _beginn;
  
  /**
   * Ende dieser Simulation
   */
  private long _ende;
  
  /**
   * Erzeuge eine neue Steuerungs-Instanz fr eine Offline-Simulation
   * @param simulation die Simulation, zu der die Instanz erzeugt werden soll
   */
  public SteuerungOfflineSimulation(Simulation simulation)
  {
    super(simulation);
    _faktor = (float)1.00;
    _beginn = 0;
    _ende = 0;
  }
  
  @Override
  public boolean erstelleWorkflow(int uebergang)
  {
    switch(uebergang)
    {
      case 0: // Neu -> Vorstart
      {
        if (super.erstelleWorkflow(0))
        {  
          WarteThread t1 = new WarteThread()
          {
            public void run()
            {
              setResult(isSimulationsZeitBereichGueltig());
            }
          };
          
          WarteThread t2 = new WarteThread();
          
          t1.start(new WarteThread[] { });
          t2.start(new WarteThread[] { t1 });
                  
          return t2.getResult();
        }
        else
        {
          return false;
        }
      }  
      
      case 1: // Vorstart -> Start
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            setResult(isKonfigurationBereit());
          }
        };

        WarteThread t2 = new WarteThread()
        {
          public void run()
          {
            setResult(isParametrierungBereit());
          }
        };
        
        WarteThread t3 = new WarteThread()
        {
          public void run()
          {
            setResult(isArchivSystemBereit());
          }
        };

        WarteThread t4 = new WarteThread()
        {
          public void run()
          {
            setResult(sendeStart());
          }
        };
        
        WarteThread t5 = new WarteThread()
        {
          public void run()
          {
            starteDatengenerator();
          }
        };
        
        WarteThread t6 = new WarteThread(); 
        
        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { });
        t3.start(new WarteThread[] { });
        t4.start(new WarteThread[] { t1, t2, t3 });
        t5.start(new WarteThread[] { t4 });
        t6.start(new WarteThread[] { t5 });
        
        return t6.getResult();
      }    
      case 2: // Start -> Stop
      case 3: // Stop -> gelscht
      case 4: // Stop -> Vorstart
        return super.erstelleWorkflow(uebergang);
        
      case 5: // Start -> Pause
      {
        {  
          WarteThread t1 = new WarteThread()
          {
            public void run()
            {
              pauseDatengenerator();
            }
          };
          
          WarteThread t2 = new WarteThread();
          
          t1.start(new WarteThread[] { });
          t2.start(new WarteThread[] { t1 });
                  
          return t2.getResult();
        }
        
      }
      case 6: // Pause -> Start
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            starteDatengenerator();
          }
        };
        
        WarteThread t2 = new WarteThread();
        
        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { t1 });
                
        return t2.getResult();
      }
      case 7: // Pause -> Pause
      {
        WarteThread t1 = new WarteThread()
        {
          public void run()
          {
            pauseDatengenerator();
          }
        };
        
        WarteThread t2 = new WarteThread();
        
        t1.start(new WarteThread[] { });
        t2.start(new WarteThread[] { t1 });
                
        return t2.getResult();
      }
      case 8: // Pause -> Stop
      {
        return erstelleWorkflow(2);
      }
    }

    return false;
  }
  
  @Override
  public boolean sendeZustand()
  {
    ClientDavInterface con = _simulation.getConnection();
    AttributeGroup atg = con.getDataModel().getAttributeGroup("atg.simulationsSteuerungOffline");
    Aspect asp = con.getDataModel().getAspect("asp.zustand");
    
    String zustand = _simulation.getZustand().toString();
    Data data = con.createData(atg);
    TextValue textValue = data.getTextValue("SimulationsZustand");
    textValue.setText(zustand);
    NumberValue nv = data.getScaledValue("Simulationsgeschwindigkeit");
    nv.set(getFaktor());
        
    _debug.info("Publiziere Zustand \"" + zustand +
        "\" (Geschwindigkeitsfaktor " + getFaktor() +
        ") fr Simulation " + _simulation);
    if (!_simulation.sendeAlsQuelle(atg, asp, data))
    {
      _debug.error("Fehler beim Senden an Simulationsobjekt " + _simulation);
     return false;
    }
    _debug.fine("Zustand Simulation " + _simulation + " erfolgreich versendet");
    return true;
  }
  
  /**
   * Sendet den Zeitbereich der Offline-Simulation an den Datenverteiler
   * @return true, wenn erfolgreich, false sonst
   */
  public boolean sendeZeitbereich()
  {
    ClientDavInterface con = _simulation.getConnection();
    AttributeGroup atg = con.getDataModel().getAttributeGroup("atg.simulationsZeitBereich");
    Aspect asp = con.getDataModel().getAspect("asp.zustand");
    
    Data data = con.createData(atg);
    
    try
    {
      TimeValue timeValue = data.getTimeValue("SimulationsBeginn");
      timeValue.setMillis(getBeginn());
      timeValue = data.getTimeValue("SimulationsEnde");
      timeValue.setMillis(getEnde());
    }
    catch(RuntimeException e)
    {
      _debug.error("Ungltige(n) Wert(e) fr Simulationsbeginn und/oder -Ende:\n" +
          e.getMessage());
      // Veranlasst das Senden von NO_DATA, da aktuell gehaltene Daten ungltig
      data = null;
      setBeginn(0);
      setEnde(0);
    }

    _debug.info("Publiziere Zeitbereich fr Simulation " + _simulation);
    try
    {
      if (!_simulation.sendeAlsQuelle(atg, asp, data))
      {
        _debug.error("Fehler beim Senden an Simulationsobjekt " + _simulation);
       return false;
      }
    }
    catch(RuntimeException e)
    {
      _debug.error("Fehler beim Senden an Simulationsobjekt " + _simulation +
          ": " + e.getMessage());
      return false;
    }
    _debug.fine("Zustand Zeitbereich Simulation " + _simulation + " erfolgreich versendet");
    return true;
  }
  
  public boolean isSimulationsZeitBereichGueltig()
  {
    int result = 0;
        
    _debug.fine("Prfe Gltigkeit Zeitbereich Offline-Simulation " + _simulation);
    
    if (!(getBeginn() > 0) && !(getEnde() > 0))
    {
      result = 1; // Beginn und Ende ungltig
    }
    else if (!(getBeginn() > 0))
    {
      result = 2; // Beginn ungltig
    }
    else if (!(getEnde() > 0))
    {
      result = 3; // Beginn ungltig
    }
    else if (!(getBeginn() < getEnde()))
    {
      result = 4; // Beginn muss kleiner als Ende sein 
    }
    else if (!(getEnde() < System.currentTimeMillis()))
    {
      result = 5; // Zeitbereich muss in der Vergangenheit liegen
    }
      
    if (result == 0)
    {  
      _debug.info("Zeitbereich ist gltig fr Offline-Simulation " + _simulation);
    }
    else
    {
      switch(result)
      {
        case 1:
          _debug.error("Beginn-und Ende sind ungltig im Zeitbereich fr Offline-Simulation " + _simulation);
          break;
        case 2:
          _debug.error("Beginn ist ungltig im Zeitbereich fr Offline-Simulation " + _simulation);
          break;
        case 3:
          _debug.error("Ende ist ungltig im Zeitbereich fr Offline-Simulation " + _simulation);
          break;
        case 4: 
          _debug.error("Ende muss spter sein als Beginn im Zeitbereich fr Offline-Simulation " + _simulation);
          break;
        case 5: 
          _debug.error("Zeitbereich fr Offline-Simulation " + _simulation + " muss in der Vergangenheit liegen");
          break;
      }   
      _debug.error("Zeitbereich ist ungltig fr Offline-Simulation " + _simulation);
    }
       
    return (result == 0);
  }

  /**
   * Bestimme den Geschwindigkeitsfaktor fr die zugeordnete Offline-Simulation
   * @return den Geschwindigkeitsfaktor fr die zugeordnete Offline-Simulation
   */
  public float getFaktor()
  {
    return _faktor;
  }
 
  /**
   * Setze den Geschwindigkeitsfaktor fr die zugeordnete Offline-Simulation
   * @param faktor der neue Geschwindigkeitsfaktor
   */
  public void setFaktor(float faktor)
  {
    _faktor = faktor;
  }
  
  /**
   * Bestimme den Beginn der zugeordneten Offline-Simulation
   * @return den aktuellen Beginn der zugeordneten Offline-Simulation
   */
  public long getBeginn()
  {
    return _beginn;
  }

  /**
   * Setze den Beginn der zugeordneten Offline-Simulation
   * @param beginn den neuen Simulations-Beginn
   */
  public void setBeginn(long beginn)
  {
    _beginn = beginn < 0 ? 0 : beginn;
  }
  
  /**
   * Bestimme das Ende der zugeordneten Offline-Simulation
   * @return das aktuelle Ende der zugeordneten Offline-Simulation
   */
  public long getEnde()
  {
    return _ende;
  }

  /**
   * Setze das Ende der zugeordneten Offline-Simulation
   * @param ende das neue Simulations-Ende
   */
  public void setEnde(long ende)
  {
    _ende = ende < 0 ? 0 : ende;
  }
  
  @Override
  protected void starteDatengenerator()
  {
    if (_generator == null)
    {
      _debug.config("Initialisiere Datengenerator fr Offline-Simulation " + _simulation);
      _generator = new DatenGeneratorOfflineSimulation((OfflineSimulation)_simulation) ;
    }  

    _debug.config("Starte Datengenerator fr Offline-Simulation " + _simulation + 
        " mit Geschwindigkeitsfaktor " + getFaktor());
    _generator.start();
  }
  
  /**
   * Setze den Datengenerator fr die zugeordnete Offline-Simulation 
   * in den Zustand "Pause"
   */
  protected void pauseDatengenerator()
  {
    Zustand zustand = _simulation.getZustand();
    
    if (zustand.getName().equals(ZustandsBezeichner.Start.text))
    {
      _debug.info("Pause fr Datengenerator der Offline-Simulation " + _simulation);
    }
    else if (zustand.getName().equals(ZustandsBezeichner.Pause.text))
    {
      _debug.info("Einzelschritt fr Datengenerator der Offline-Simulation " + _simulation);
    }
    
    ((DatenGeneratorOfflineSimulation)_generator).pause();
  }
  
  @Override
  protected void beendeDatengenerator()
  {
    _debug.info("Beende Datengenerator fr Offline-Simulation " + _simulation);
    
    _generator.stop();
    _generator = null;
  }
  
}
